-------------------------------------------------------------------------------
-- Xavante configuration file.
--
-- The configuration defines the Xavante environment
-- Commented fields are optional and assume default values
--
-- Xavante defines virtualhosts for each site running with Xavante.
-- Each virtualhost can define the handlers for specific files extensions.
-- Xavante currently offers a fileHandler and a CGILuaHandler.
--
-- Xavante configuration can be redefined on the default file structure by the
-- the use of LUA_PATH, see more details in the online documentation.
--
-- Authors: Javier Guerra and Andre Carregal
-- Copyright (c) 2004-2006 Kepler Project
---
-- $Id: config.lua,v 1.3 2011/12/12 12:18:49 mtamarsky Exp $
------------------------------------------------------------------------------
--wsapi = require"wsapi"
xavante = require"xavante"
filehandler = require "xavante.filehandler"
cgiluahandler = require "xavante.cgiluahandler"
redirecthandler = require "xavante.redirecthandler"
indexhandler = require "xavante.indexhandler"

-- Define here where Xavante HTTP documents scripts are located
local webDir = XAVANTE_WEB

local simplerules = {   
    {
      match = "^/help/(.*)$",
      with = redirecthandler,
      params = {"/nsghelpcd.lp?helppath=%1"}
    },

    { -- URI remapping example
      match = "^/$",
      with = redirecthandler,
      params = {"index.lp"}
    }, 

    { -- cgiluahandler example
      match = {"%.lp$", "%.lp/.*$", "%.lua$", "%.lua/.*$" },
      with = cgiluahandler.makeHandler (webDir)
    },

    
    { -- filehandler example
      match = ".",
      with = filehandler,
      params = {baseDir = webDir}
    },
} 

-- Displays a message in the console with the used ports
xavante.start_message(function (ports)
    local date = os.date("[%Y-%m-%d %H:%M:%S]")
    print(string.format("%s Xavante started on port(s) %s",
      date, table.concat(ports, ", ")))
  end)

xavante.HTTP{  --Zmey NSG
    server = {host = "*", port = NSGCONFD_HTTP_PORT or 80},
    
    defaultHost = {
        rules = simplerules
    },
}
